import FWCore.ParameterSet.Config as cms

from PhysicsTools.SelectorUtils.centralIDRegistry import central_id_registry

ebCutOff = 1.479

#
# The ID cuts below are optimized IDs for CSA14 Scenario 1 (50ns, with misalignment)
# The cut values are taken from the twiki:
#       https://twiki.cern.ch/twiki/bin/view/CMS/CutBasedElectronIdentificationRun2
#       (where they may not stay, if a newer version of cuts becomes available for these
#        conditions)
# See also the presentation explaining these working points (this will not change):
#        https://indico.cern.ch/event/298244/contribution/1/material/slides/1.pdf

cutBasedElectronID_CSA14_50ns_V1_standalone_veto = cms.PSet(
    idName = cms.string("cutBasedElectronID-CSA14-50ns-V1-standalone-veto"),
    cutFlow = cms.VPSet(
        cms.PSet( cutName = cms.string("MinPtCut"),
                  minPt = cms.double(5.0),
                  needsAdditionalProducts = cms.bool(False),
                  isIgnored = cms.bool(False)                ),
        cms.PSet( cutName = cms.string("GsfEleSCEtaMultiRangeCut"),
                  useAbsEta = cms.bool(True),
                  allowedEtaRanges = cms.VPSet( 
                                  cms.PSet( minEta = cms.double(0.0), 
                                            maxEta = cms.double(ebCutOff) ),
                                  cms.PSet( minEta = cms.double(ebCutOff), 
                                            maxEta = cms.double(2.5) )
                                  ),
                  needsAdditionalProducts = cms.bool(False),
                  isIgnored = cms.bool(False)),
        cms.PSet( cutName = cms.string('GsfEleDEtaInCut'),
                  dEtaInCutValueEB = cms.double(0.021156),
                  dEtaInCutValueEE = cms.double(0.028286),
                  barrelCutOff = cms.double(ebCutOff),
                  needsAdditionalProducts = cms.bool(False),
                  isIgnored = cms.bool(False)),
        cms.PSet( cutName = cms.string('GsfEleDPhiInCut'),
                  dPhiInCutValueEB = cms.double(0.247197),
                  dPhiInCutValueEE = cms.double(0.228815),
                  barrelCutOff = cms.double(ebCutOff),
                  needsAdditionalProducts = cms.bool(False),
                  isIgnored = cms.bool(False)),
        cms.PSet( cutName = cms.string('GsfEleFull5x5SigmaIEtaIEtaCut'),
                  full5x5SigmaIEtaIEtaCutValueEB = cms.double(0.012236),
                  full5x5SigmaIEtaIEtaCutValueEE = cms.double(0.035176),
                  full5x5SigmaIEtaIEtaMap = cms.InputTag('electronIDValueMapProducer:eleFull5x5SigmaIEtaIEta'),
                  barrelCutOff = cms.double(ebCutOff),
                  needsAdditionalProducts = cms.bool(False),
                  isIgnored = cms.bool(False)),
        cms.PSet( cutName = cms.string('GsfEleHadronicOverEMCut'),
                  hadronicOverEMCutValueEB = cms.double(0.241641),
                  hadronicOverEMCutValueEE = cms.double(0.185910),
                  barrelCutOff = cms.double(ebCutOff),
                  needsAdditionalProducts = cms.bool(False),
                  isIgnored = cms.bool(False)),
        cms.PSet( cutName = cms.string('GsfEleDxyCut'),
                  dxyCutValueEB = cms.double(0.031812),
                  dxyCutValueEE = cms.double(0.216331),
                  vertexSrc = cms.InputTag("offlinePrimaryVertices"),
                  vertexSrcMiniAOD = cms.InputTag("offlineSlimmedPrimaryVertices"),
                  barrelCutOff = cms.double(ebCutOff),
                  needsAdditionalProducts = cms.bool(True),
                  isIgnored = cms.bool(False)),
        cms.PSet( cutName = cms.string('GsfEleDzCut'),
                  dzCutValueEB = cms.double(0.499344),
                  dzCutValueEE = cms.double(0.911467),
                  vertexSrc = cms.InputTag("offlinePrimaryVertices"),
                  vertexSrcMiniAOD = cms.InputTag("offlineSlimmedPrimaryVertices"),
                  barrelCutOff = cms.double(ebCutOff),
                  needsAdditionalProducts = cms.bool(True),
                  isIgnored = cms.bool(False)),
        cms.PSet( cutName = cms.string('GsfEleEInverseMinusPInverseCut'),
                  eInverseMinusPInverseCutValueEB = cms.double(0.323747),
                  eInverseMinusPInverseCutValueEE = cms.double(0.133209),
                  barrelCutOff = cms.double(ebCutOff),
                  needsAdditionalProducts = cms.bool(False),
                  isIgnored = cms.bool(False)),
        cms.PSet( cutName = cms.string('GsfEleDeltaBetaIsoCutStandalone'),
                  isoCutEBLowPt = cms.double(0.239832),
                  isoCutEBHighPt = cms.double(0.239832),
                  isoCutEELowPt = cms.double(0.237643),
                  isoCutEEHighPt = cms.double(0.237643),
                  isRelativeIso = cms.bool(True),
                  deltaBetaConstant = cms.double(0.5),
                  ptCutOff = cms.double(20.0),
                  barrelCutOff = cms.double(ebCutOff),
                  needsAdditionalProducts = cms.bool(False),
                  isIgnored = cms.bool(False)),
        cms.PSet( cutName = cms.string('GsfEleConversionVetoCut'),
                  conversionSrc = cms.InputTag('allConversions'),
                  conversionSrcMiniAOD = cms.InputTag('reducedEgamma:reducedConversions'),
                  beamspotSrc = cms.InputTag('offlineBeamSpot'),
                  needsAdditionalProducts = cms.bool(True),
                  isIgnored = cms.bool(False)),
        cms.PSet( cutName = cms.string('GsfEleMissingHitsCut'),
                  maxMissingHitsEB = cms.uint32(2),
                  maxMissingHitsEE = cms.uint32(3),
                  barrelCutOff = cms.double(ebCutOff),
                  needsAdditionalProducts = cms.bool(False),
                  isIgnored = cms.bool(False))                   
    )
)

cutBasedElectronID_CSA14_50ns_V1_standalone_loose = cms.PSet(
    idName = cms.string("cutBasedElectronID-CSA14-50ns-V1-standalone-loose"),
    cutFlow = cms.VPSet(
        cms.PSet( cutName = cms.string("MinPtCut"),
                  minPt = cms.double(5.0),
                  needsAdditionalProducts = cms.bool(False),
                  isIgnored = cms.bool(False)                ),
        cms.PSet( cutName = cms.string("GsfEleSCEtaMultiRangeCut"),
                  useAbsEta = cms.bool(True),
                  allowedEtaRanges = cms.VPSet( 
                                  cms.PSet( minEta = cms.double(0.0), 
                                            maxEta = cms.double(ebCutOff) ),
                                  cms.PSet( minEta = cms.double(ebCutOff), 
                                            maxEta = cms.double(2.5) )
                                  ),
                  needsAdditionalProducts = cms.bool(False),
                  isIgnored = cms.bool(False)),
        cms.PSet( cutName = cms.string('GsfEleDEtaInCut'),
                  dEtaInCutValueEB = cms.double(0.015792),
                  dEtaInCutValueEE = cms.double(0.024830),
                  barrelCutOff = cms.double(ebCutOff),
                  needsAdditionalProducts = cms.bool(False),
                  isIgnored = cms.bool(False)),
        cms.PSet( cutName = cms.string('GsfEleDPhiInCut'),
                  dPhiInCutValueEB = cms.double(0.079938),
                  dPhiInCutValueEE = cms.double(0.096950),
                  barrelCutOff = cms.double(ebCutOff),
                  needsAdditionalProducts = cms.bool(False),
                  isIgnored = cms.bool(False)),
        cms.PSet( cutName = cms.string('GsfEleFull5x5SigmaIEtaIEtaCut'),
                  full5x5SigmaIEtaIEtaCutValueEB = cms.double(0.011836),
                  full5x5SigmaIEtaIEtaCutValueEE = cms.double(0.032479),
                  full5x5SigmaIEtaIEtaMap = cms.InputTag('electronIDValueMapProducer:eleFull5x5SigmaIEtaIEta'),
                  barrelCutOff = cms.double(ebCutOff),
                  needsAdditionalProducts = cms.bool(False),
                  isIgnored = cms.bool(False)),
        cms.PSet( cutName = cms.string('GsfEleHadronicOverEMCut'),
                  hadronicOverEMCutValueEB = cms.double(0.145705),
                  hadronicOverEMCutValueEE = cms.double(0.115660),
                  barrelCutOff = cms.double(ebCutOff),
                  needsAdditionalProducts = cms.bool(False),
                  isIgnored = cms.bool(False)),
        cms.PSet( cutName = cms.string('GsfEleDxyCut'),
                  dxyCutValueEB = cms.double(0.019094),
                  dxyCutValueEE = cms.double(0.098601),
                  vertexSrc = cms.InputTag("offlinePrimaryVertices"),
                  vertexSrcMiniAOD = cms.InputTag("offlineSlimmedPrimaryVertices"),
                  barrelCutOff = cms.double(ebCutOff),
                  needsAdditionalProducts = cms.bool(True),
                  isIgnored = cms.bool(False)),
        cms.PSet( cutName = cms.string('GsfEleDzCut'),
                  dzCutValueEB = cms.double(0.035802),
                  dzCutValueEE = cms.double(0.879581),
                  vertexSrc = cms.InputTag("offlinePrimaryVertices"),
                  vertexSrcMiniAOD = cms.InputTag("offlineSlimmedPrimaryVertices"),
                  barrelCutOff = cms.double(ebCutOff),
                  needsAdditionalProducts = cms.bool(True),
                  isIgnored = cms.bool(False)),
        cms.PSet( cutName = cms.string('GsfEleEInverseMinusPInverseCut'),
                  eInverseMinusPInverseCutValueEB = cms.double(0.105459),
                  eInverseMinusPInverseCutValueEE = cms.double(0.110190),
                  barrelCutOff = cms.double(ebCutOff),
                  needsAdditionalProducts = cms.bool(False),
                  isIgnored = cms.bool(False)),
        cms.PSet( cutName = cms.string('GsfEleDeltaBetaIsoCutStandalone'),
                  isoCutEBLowPt = cms.double(0.183981),
                  isoCutEBHighPt = cms.double(0.183981),
                  isoCutEELowPt = cms.double(0.211836),
                  isoCutEEHighPt = cms.double(0.211836),
                  isRelativeIso = cms.bool(True),
                  deltaBetaConstant = cms.double(0.5),
                  ptCutOff = cms.double(20.0),
                  barrelCutOff = cms.double(ebCutOff),
                  needsAdditionalProducts = cms.bool(False),
                  isIgnored = cms.bool(False)),
        cms.PSet( cutName = cms.string('GsfEleConversionVetoCut'),
                  conversionSrc = cms.InputTag('allConversions'),
                  conversionSrcMiniAOD = cms.InputTag('reducedEgamma:reducedConversions'),
                  beamspotSrc = cms.InputTag('offlineBeamSpot'),
                  needsAdditionalProducts = cms.bool(True),
                  isIgnored = cms.bool(False)),
        cms.PSet( cutName = cms.string('GsfEleMissingHitsCut'),
                  maxMissingHitsEB = cms.uint32(1),
                  maxMissingHitsEE = cms.uint32(1),
                  barrelCutOff = cms.double(ebCutOff),
                  needsAdditionalProducts = cms.bool(False),
                  isIgnored = cms.bool(False))                   
    )
)

cutBasedElectronID_CSA14_50ns_V1_standalone_medium = cms.PSet(
    idName = cms.string("cutBasedElectronID-CSA14-50ns-V1-standalone-medium"),
    cutFlow = cms.VPSet(
        cms.PSet( cutName = cms.string("MinPtCut"),
                  minPt = cms.double(5.0),
                  needsAdditionalProducts = cms.bool(False),
                  isIgnored = cms.bool(False)                ),
        cms.PSet( cutName = cms.string("GsfEleSCEtaMultiRangeCut"),
                  useAbsEta = cms.bool(True),
                  allowedEtaRanges = cms.VPSet( 
                                  cms.PSet( minEta = cms.double(0.0), 
                                            maxEta = cms.double(ebCutOff) ),
                                  cms.PSet( minEta = cms.double(ebCutOff), 
                                            maxEta = cms.double(2.5) )
                                  ),
                  needsAdditionalProducts = cms.bool(False),
                  isIgnored = cms.bool(False)),
        cms.PSet( cutName = cms.string('GsfEleDEtaInCut'),
                  dEtaInCutValueEB = cms.double(0.015382),
                  dEtaInCutValueEE = cms.double(0.022962),
                  barrelCutOff = cms.double(ebCutOff),
                  needsAdditionalProducts = cms.bool(False),
                  isIgnored = cms.bool(False)),
        cms.PSet( cutName = cms.string('GsfEleDPhiInCut'),
                  dPhiInCutValueEB = cms.double(0.050766),
                  dPhiInCutValueEE = cms.double(0.056415),
                  barrelCutOff = cms.double(ebCutOff),
                  needsAdditionalProducts = cms.bool(False),
                  isIgnored = cms.bool(False)),
        cms.PSet( cutName = cms.string('GsfEleFull5x5SigmaIEtaIEtaCut'),
                  full5x5SigmaIEtaIEtaCutValueEB = cms.double(0.010326),
                  full5x5SigmaIEtaIEtaCutValueEE = cms.double(0.030011),
                  full5x5SigmaIEtaIEtaMap = cms.InputTag('electronIDValueMapProducer:eleFull5x5SigmaIEtaIEta'),
                  barrelCutOff = cms.double(ebCutOff),
                  needsAdditionalProducts = cms.bool(False),
                  isIgnored = cms.bool(False)),
        cms.PSet( cutName = cms.string('GsfEleHadronicOverEMCut'),
                  hadronicOverEMCutValueEB = cms.double(0.100644),
                  hadronicOverEMCutValueEE = cms.double(0.099253),
                  barrelCutOff = cms.double(ebCutOff),
                  needsAdditionalProducts = cms.bool(False),
                  isIgnored = cms.bool(False)),
        cms.PSet( cutName = cms.string('GsfEleDxyCut'),
                  dxyCutValueEB = cms.double(0.012012),
                  dxyCutValueEE = cms.double(0.067921),
                  vertexSrc = cms.InputTag("offlinePrimaryVertices"),
                  vertexSrcMiniAOD = cms.InputTag("offlineSlimmedPrimaryVertices"),
                  barrelCutOff = cms.double(ebCutOff),
                  needsAdditionalProducts = cms.bool(True),
                  isIgnored = cms.bool(False)),
        cms.PSet( cutName = cms.string('GsfEleDzCut'),
                  dzCutValueEB = cms.double(0.030398),
                  dzCutValueEE = cms.double(0.781003),
                  vertexSrc = cms.InputTag("offlinePrimaryVertices"),
                  vertexSrcMiniAOD = cms.InputTag("offlineSlimmedPrimaryVertices"),
                  barrelCutOff = cms.double(ebCutOff),
                  needsAdditionalProducts = cms.bool(True),
                  isIgnored = cms.bool(False)),
        cms.PSet( cutName = cms.string('GsfEleEInverseMinusPInverseCut'),
                  eInverseMinusPInverseCutValueEB = cms.double(0.052553),
                  eInverseMinusPInverseCutValueEE = cms.double(0.109146),
                  barrelCutOff = cms.double(ebCutOff),
                  needsAdditionalProducts = cms.bool(False),
                  isIgnored = cms.bool(False)),
        cms.PSet( cutName = cms.string('GsfEleDeltaBetaIsoCutStandalone'),
                  isoCutEBLowPt = cms.double(0.135355),
                  isoCutEBHighPt = cms.double(0.135355),
                  isoCutEELowPt = cms.double(0.146530),
                  isoCutEEHighPt = cms.double(0.146530),
                  isRelativeIso = cms.bool(True),
                  deltaBetaConstant = cms.double(0.5),
                  ptCutOff = cms.double(20.0),
                  barrelCutOff = cms.double(ebCutOff),
                  needsAdditionalProducts = cms.bool(False),
                  isIgnored = cms.bool(False)),
        cms.PSet( cutName = cms.string('GsfEleConversionVetoCut'),
                  conversionSrc = cms.InputTag('allConversions'),
                  conversionSrcMiniAOD = cms.InputTag('reducedEgamma:reducedConversions'),
                  beamspotSrc = cms.InputTag('offlineBeamSpot'),
                  needsAdditionalProducts = cms.bool(True),
                  isIgnored = cms.bool(False)),
        cms.PSet( cutName = cms.string('GsfEleMissingHitsCut'),
                  maxMissingHitsEB = cms.uint32(1),
                  maxMissingHitsEE = cms.uint32(1),
                  barrelCutOff = cms.double(ebCutOff),
                  needsAdditionalProducts = cms.bool(False),
                  isIgnored = cms.bool(False))                   
    )
)

cutBasedElectronID_CSA14_50ns_V1_standalone_tight = cms.PSet(
    idName = cms.string("cutBasedElectronID-CSA14-50ns-V1-standalone-tight"),
    cutFlow = cms.VPSet(
        cms.PSet( cutName = cms.string("MinPtCut"),
                  minPt = cms.double(5.0),
                  needsAdditionalProducts = cms.bool(False),
                  isIgnored = cms.bool(False)                ),
        cms.PSet( cutName = cms.string("GsfEleSCEtaMultiRangeCut"),
                  useAbsEta = cms.bool(True),
                  allowedEtaRanges = cms.VPSet( 
                                  cms.PSet( minEta = cms.double(0.0), 
                                            maxEta = cms.double(ebCutOff) ),
                                  cms.PSet( minEta = cms.double(ebCutOff), 
                                            maxEta = cms.double(2.5) )
                                  ),
                  needsAdditionalProducts = cms.bool(False),
                  isIgnored = cms.bool(False)),
        cms.PSet( cutName = cms.string('GsfEleDEtaInCut'),
                  dEtaInCutValueEB = cms.double(0.012316),
                  dEtaInCutValueEE = cms.double(0.019221),
                  barrelCutOff = cms.double(ebCutOff),
                  needsAdditionalProducts = cms.bool(False),
                  isIgnored = cms.bool(False)),
        cms.PSet( cutName = cms.string('GsfEleDPhiInCut'),
                  dPhiInCutValueEB = cms.double(0.024063),
                  dPhiInCutValueEE = cms.double(0.042714),
                  barrelCutOff = cms.double(ebCutOff),
                  needsAdditionalProducts = cms.bool(False),
                  isIgnored = cms.bool(False)),
        cms.PSet( cutName = cms.string('GsfEleFull5x5SigmaIEtaIEtaCut'),
                  full5x5SigmaIEtaIEtaCutValueEB = cms.double(0.010244),
                  full5x5SigmaIEtaIEtaCutValueEE = cms.double(0.028612),
                  full5x5SigmaIEtaIEtaMap = cms.InputTag('electronIDValueMapProducer:eleFull5x5SigmaIEtaIEta'),
                  barrelCutOff = cms.double(ebCutOff),
                  needsAdditionalProducts = cms.bool(False),
                  isIgnored = cms.bool(False)),
        cms.PSet( cutName = cms.string('GsfEleHadronicOverEMCut'),
                  hadronicOverEMCutValueEB = cms.double(0.073594),
                  hadronicOverEMCutValueEE = cms.double(0.080394),
                  barrelCutOff = cms.double(ebCutOff),
                  needsAdditionalProducts = cms.bool(False),
                  isIgnored = cms.bool(False)),
        cms.PSet( cutName = cms.string('GsfEleDxyCut'),
                  dxyCutValueEB = cms.double(0.009088),
                  dxyCutValueEE = cms.double(0.037208),
                  vertexSrc = cms.InputTag("offlinePrimaryVertices"),
                  vertexSrcMiniAOD = cms.InputTag("offlineSlimmedPrimaryVertices"),
                  barrelCutOff = cms.double(ebCutOff),
                  needsAdditionalProducts = cms.bool(True),
                  isIgnored = cms.bool(False)),
        cms.PSet( cutName = cms.string('GsfEleDzCut'),
                  dzCutValueEB = cms.double(0.016881),
                  dzCutValueEE = cms.double(0.064947),
                  vertexSrc = cms.InputTag("offlinePrimaryVertices"),
                  vertexSrcMiniAOD = cms.InputTag("offlineSlimmedPrimaryVertices"),
                  barrelCutOff = cms.double(ebCutOff),
                  needsAdditionalProducts = cms.bool(True),
                  isIgnored = cms.bool(False)),
        cms.PSet( cutName = cms.string('GsfEleEInverseMinusPInverseCut'),
                  eInverseMinusPInverseCutValueEB = cms.double(0.025730),
                  eInverseMinusPInverseCutValueEE = cms.double(0.076298),
                  barrelCutOff = cms.double(ebCutOff),
                  needsAdditionalProducts = cms.bool(False),
                  isIgnored = cms.bool(False)),
        cms.PSet( cutName = cms.string('GsfEleDeltaBetaIsoCutStandalone'),
                  isoCutEBLowPt = cms.double(0.100254),
                  isoCutEBHighPt = cms.double(0.100254),
                  isoCutEELowPt = cms.double(0.137026),
                  isoCutEEHighPt = cms.double(0.137026),
                  isRelativeIso = cms.bool(True),
                  deltaBetaConstant = cms.double(0.5),
                  ptCutOff = cms.double(20.0),
                  barrelCutOff = cms.double(ebCutOff),
                  needsAdditionalProducts = cms.bool(False),
                  isIgnored = cms.bool(False)),
        cms.PSet( cutName = cms.string('GsfEleConversionVetoCut'),
                  conversionSrc = cms.InputTag('allConversions'),
                  conversionSrcMiniAOD = cms.InputTag('reducedEgamma:reducedConversions'),
                  beamspotSrc = cms.InputTag('offlineBeamSpot'),
                  needsAdditionalProducts = cms.bool(True),
                  isIgnored = cms.bool(False)),
        cms.PSet( cutName = cms.string('GsfEleMissingHitsCut'),
                  maxMissingHitsEB = cms.uint32(1),
                  maxMissingHitsEE = cms.uint32(1),
                  barrelCutOff = cms.double(ebCutOff),
                  needsAdditionalProducts = cms.bool(False),
                  isIgnored = cms.bool(False))                   
    )
)

central_id_registry.register(cutBasedElectronID_CSA14_50ns_V1_standalone_veto.idName,
                             '0443b6d07f602b4ee4ccabd851e4d364')
central_id_registry.register(cutBasedElectronID_CSA14_50ns_V1_standalone_loose.idName,
                             '078717cbe5967273b808379cf4319e6c')
central_id_registry.register(cutBasedElectronID_CSA14_50ns_V1_standalone_medium.idName,
                             '6ae65b48e6f5efcc468fcad883a599fb')
central_id_registry.register(cutBasedElectronID_CSA14_50ns_V1_standalone_tight.idName,
                             '767014939fe5066f79e4d9b9dc92bb09')









